#!/usr/bin/env bash
set -e

_log(){
    now=$(date)
    set +e
    echo "$now - $1 - $2"
    set -e
}

_log_info(){
    _log "INFO" "$1"
}

_log_warn(){
    _log "WARN" "$1"
}

_log_error(){
    _log "ERROR" "$1"
}

_log_info "Start setting up kserve"

dir="$( cd -- "$(dirname "$0")" >/dev/null 2>&1 ; pwd -P )"

source $dir/variables.sh

mkdir -p $ASSETS_DIR

pushd $ASSETS_DIR

_log_info "Setting up Docker images"

## Setup Docker images

if [ ! -e $ASSETS_DIR/docker-images-$HOPSWORKS_VERSION.tar ];
then
    curl -L https://repo.hops.works/master/kube/docker-images/${HOPSWORKS_VERSION}/docker-images.tar -o $ASSETS_DIR/docker-images-$HOPSWORKS_VERSION.tar
fi

docker load < $ASSETS_DIR/docker-images-$HOPSWORKS_VERSION.tar

k8s_docker_images=("model-serving-webhook"
		   "storage-initializer"
		   "model-serving-authenticator"
		   "inference-logger"
		   "sklearnserver")

for i in ${k8s_docker_images[@]}; do
    docker tag docker.hops.works/$i:$HOPSWORKS_VERSION $DOCKER_REGISTRY_URL/$i:$HOPSWORKS_VERSION
    docker push $DOCKER_REGISTRY_URL/$i:$HOPSWORKS_VERSION
done

_log_info "Finished with Docker images"

_log_info "Downloading istio $ISTIO_VERSION"
mkdir -p $BIN_DIR
rm -rf ${ISTIO_DIR}
mkdir -p $ISTIO_DIR
if [ ! -e istio-$ISTIO_VERSION ];
then
    curl -L https://istio.io/downloadIstio | ISTIO_VERSION=$ISTIO_VERSION sh -
fi
mv istio-$ISTIO_VERSION $BIN_DIR

if [ "$INSTALL_KUBECTL" == "true" ];
then
    _log_info "Installing kubectl"
    curl -L https://dl.k8s.io/release/v${$KUBERNETES_FAMILY_VERSION}.0/bin/linux/amd64/kubectl -o ${BIN_DIR}/kubectl && chmod +x ${BIN_DIR}/kubectl
else
    _log_warn "Skip installing kubectl becase INSTALL_KUBECTL=$INSTALL_KUBECTL"
fi

_log_info "Templating hops-system.yml"
envsubst < $dir/templates/hops-system.yml > hops-system.yml
_log_info "Applying hops-system.yml"
$KUBECTL apply -f hops-system.yml

_log_info "Templating istio-operator.yml"
envsubst < $dir/templates/istio-operator.yml > istio-operator.yml
_log_info "Configuring Istio"
$ISTIO_DIR/bin/istioctl install --skip-confirmation -f istio-operator.yml

_log_info "Setting Envoy filter"
$KUBECTL apply -f $dir/files/istio-envoy-filter.yml

_log_info "Setting Knative CRDS"
$KUBECTL apply -f $dir/files/knative-serving-crds.yml

envsubst < $dir/templates/knative-serving.yml > knative-serving.yml
_log_info "Setting Knative serving"
$KUBECTL apply -f knative-serving.yml -f $dir/files/knative-istio.yml

_log_info "Setting cert-manager"
$KUBECTL apply -f $dir/files/cert-manager.yml

sleep 60 

_log_info "Setting kserve"
$KUBECTL apply -f $dir/files/kserve.yml

_log_info "Setting serving runtimes"
envsubst < $dir/templates/kserve-serving-runtimes.yml > kserve-serving-runtimes.yml
$KUBECTL apply -f kserve-serving-runtimes.yml

_log_info "Generating Model serving webhook certificate"
envsubst < $dir/templates/generate_ms_webhook_cert.sh > generate_ms_webhook_cert.sh
source generate_ms_webhook_cert.sh

$KUBECTL -n hops-system create secret tls model-serving-webhook-tls --cert model-serving-webhook.crt --key model-serving-webhook.key --dry-run=client --output=yaml > model-serving-webhook-tls.yml

_log_info "Setting Model serving webhook"
export MODEL_SERVING_CERT_B64="$(cat $MODEL_SERVING_CERT_BASE64_FILE)"
envsubst < $dir/templates/model-serving-webhook.yml > model-serving-webhook.yml
$KUBECTL apply -f model-serving-webhook-tls.yml -f model-serving-webhook.yml

_log_info "Setting Model serving authenticator"
envsubst < $dir/templates/model-serving-authenticator.yml > model-serving-authenticator.yml
$KUBECTL apply -f model-serving-authenticator.yml

_log_info "Setting logstash"
envsubst < $dir/templates/filebeat-serving.yml > filebeat-serving.yml
$KUBECTL apply -f filebeat-serving.yml

_log_info "Setting Kubernetes state metrics"
$KUBECTL apply -f $dir/files/kube-state-metrics.yml

_log_info "Setting hopsmon RBAC rules"
$KUBECTL apply -f $dir/files/hopsmon-rbac.yml

_log_info "Setting Prometheus"

export MYSQL_CLIENT=/srv/hops/mysql-cluster/ndb/scripts/mysql-client.sh
envsubst < $dir/templates/prometheus_scrape_configs.yml > prometheus_scrape_configs.yml
envsubst < $dir/templates/setup_prometheus.sh > setup_prometheus.sh

source setup_prometheus.sh

_log_info "Congratulations you have setup kserve :)"
